/*
 * FIG : Facility for Interactive Generation of figures
 * Copyright (c) 1985 by Supoj Sutanthavibul
 *
 * "Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of M.I.T. not be used in advertising or
 * publicity pertaining to distribution of the software without specific,
 * written prior permission.  M.I.T. makes no representations about the
 * suitability of this software for any purpose.  It is provided "as is"
 * without express or implied warranty."
 *
 */

extern XImage	addpt_ic;

#ifdef notdef			/* not used yet */
extern XImage	raise_ic;
extern XImage	kbd_ic;

#endif
extern XImage	smartoff_ic;
extern XImage	smartmove_ic;
extern XImage	smartslide_ic;
extern XImage	arc_ic;
extern XImage	autoarrow_ic;
extern XImage	backarrow_ic;
extern XImage	box_ic;
extern XImage	regpoly_ic;
extern XImage	epsobj_ic;
extern XImage	arc_box_ic;
extern XImage	break_ic;
extern XImage	cirrad_ic;
extern XImage	cirdia_ic;
extern XImage	c_spl_ic;
extern XImage	copy_ic;
extern XImage	glue_ic;
extern XImage	solidline_ic;
extern XImage	dashline_ic;
extern XImage	dottedline_ic;
extern XImage	deletept_ic;
extern XImage	ellrad_ic;
extern XImage	elldia_ic;
extern XImage	flip_x_ic;
extern XImage	flip_y_ic;
extern XImage	forarrow_ic;
extern XImage	grid1_ic;
extern XImage	grid2_ic;
extern XImage	intspl_ic;
extern XImage	c_intspl_ic;
extern XImage	line_ic;
extern XImage	fine_grid_ic;
extern XImage	unconstrained_ic;
extern XImage	latexline_ic;
extern XImage	latexarrow_ic;
extern XImage	mounthattan_ic;
extern XImage	manhattan_ic;
extern XImage	mountain_ic;
extern XImage	move_ic;
extern XImage	movept_ic;
extern XImage	polygon_ic;
extern XImage	delete_ic;
extern XImage	rotCW_ic;
extern XImage	rotCCW_ic;
extern XImage	scale_ic;
extern XImage	convert_ic;
extern XImage	spl_ic;
extern XImage	text_ic;
extern XImage	update_ic;
extern XImage	change_ic;
extern XImage	halignl_ic;
extern XImage	halignr_ic;
extern XImage	halignc_ic;
extern XImage	valignt_ic;
extern XImage	valignb_ic;
extern XImage	valignc_ic;
extern XImage	align_ic;
extern XImage	any_ic;
extern XImage	none_ic;
extern XImage	fill_ic;
extern XImage	blank_ic;
extern XImage	textL_ic;
extern XImage	textC_ic;
extern XImage	textR_ic;
extern XImage	printer_ic;
extern XImage	noarrows_ic;
extern XImage	farrows_ic;
extern XImage	barrows_ic;
extern XImage	fbarrows_ic;
